/*
 * MIT License
 *
 * Copyright (c) 2017 Meilhaus Electronic GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <linux/serial.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

int setRS485(int fd, int enable) {
	struct serial_rs485 rs485conf;

	int status;

	//Check Enable
	if (enable)
		/* Enable RS485 mode: */
		rs485conf.flags |= SER_RS485_ENABLED;
	else
		/* Disable RS485 mode: */
		rs485conf.flags &= ~SER_RS485_ENABLED;

	//Set RS485 status
	if (ioctl(fd, TIOCSRS485, &rs485conf) < 0) {
		perror("setRS485(): TIOCSRS485");
		return 0;
	}

	return 1;
}

int main() {
	int fd;
	char *serialdev = "/dev/ttyME0";

	if ((fd = open(serialdev, O_RDWR)) == -1) {
		perror("Open port");
		return 1;
	}

	//Enable RS485 mode (automatically enables/disables RS485 transceiver before/after send)
	setRS485(fd, 1);

	/* Use read() and write() syscalls here... */

	if (close(fd) < 0)
		perror("Close port");

	return 0;
}
