#include <unistd.h>
#include <stdio.h>
#include <ctype.h>


/* This must be included in order
   to have access to the ME-4000
   specific definitions */
#include <libme9000mio.h>



void usage(void){
	printf("dioByte - Program to test digital I/O\n\n");
	printf("Usage: dioByte [arguments]\n\n");
	printf("Arguments:\n\n");
	printf("-h                     Print this help and exit.\n");
	printf("-b <board number>      Use <board number> board (Default is 0).\n");
	printf("-w <hexvalue>          Configure port for writing and write <hexvalue> to it.\n");
	printf("                       If this option is omitted the port is read.\n");
}



int main(int argc, char *argv[]){
	int err;
	char errStr[256] = {0};
	int c;
	unsigned short usBoardVersion;
	unsigned long ulSerialNumber;
	unsigned int board = 0;
	int dir = ME9000MIO_DIO_INPUT;
	unsigned char value = 0;

	/* Parse the command line arguments */
	while((c = getopt(argc, argv, "hb:w:")) != -1){
		switch(c){
			case 'h':
				usage();
				exit(0);
			case 'b':
				board = atoi(optarg);
				break;
			case 'w':
				dir = ME9000MIO_DIO_OUTPUT;
				if(sscanf(optarg, "%hhx", &value) != 1){
					usage();
					exit(2);
				}
				break;
			default:
				usage();
				exit(2);
		}
	}

	if(optind != argc){
		fprintf(stderr, "%s: No non option arguments are supported.\n", argv[0]);
		usage();
		exit(2);
	}

	printf("Open digital I/O on board %u.\n", board);
	err = me9000mioDIOOpen(board);
	if(err){
		me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
		fprintf(stderr, "ERROR: %s.\n", errStr);
		exit(1);
	}

	err = me9000mioGetBoardVersion(board, &usBoardVersion);
	if(err){
		me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
		fprintf(stderr, "ERROR: %s.\n", errStr);
		exit(1);
	}
	printf("Board Version = ME-%hu\n", usBoardVersion);

	err = me9000mioGetSerialNumber(board, &ulSerialNumber);
	if(err){
		me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
		fprintf(stderr, "ERROR: %s.\n", errStr);
		exit(1);
	}
	printf("Board Version = ME-%lu\n", ulSerialNumber);

	if(dir == ME9000MIO_DIO_OUTPUT){ // Output selected
		printf("Configure all lines as output.\n");
		for(c = 0; c < 8; c++){
			err = me9000mioDIOConfig(
					board,
					c, 
					dir,
					ME9000MIO_DIO_INV_DISABLE,
					ME9000MIO_DIO_INT_DISABLE);
			if(err){
				me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
				fprintf(stderr, "ERROR: %s.\n", errStr);
				exit(1);
			}
		}

		printf("Write 0x%X to digital output port\n", value);
		err = me9000mioDIOSetByte(board, value);
		if(err){
			me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
			fprintf(stderr, "ERROR: %s.\n", errStr);
			exit(1);
		}
	}
	else{ // Input selected
		printf("Configure all lines as input.\n");
		for(c = 0; c < 8; c++){
			err = me9000mioDIOConfig(
					board,
					c, 
					dir,
					ME9000MIO_DIO_INV_DISABLE,
					ME9000MIO_DIO_INT_DISABLE);
			if(err){
				me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
				fprintf(stderr, "ERROR: %s.\n", errStr);
				exit(1);
			}
		}

		printf("Read value from digital input port.\n");
		err = me9000mioDIOGetByte(board, &value);
		if(err){
			me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
			fprintf(stderr, "ERROR: %s.\n", errStr);
			exit(1);
		}
		printf("Value from digital input port is 0x%X.\n", value);
	}

	printf("Close digital I/O on board %u.\n", board);
	err = me9000mioDIOClose(board);
	if(err){
		me9000mioErrorGetMessage(err, errStr, sizeof(errStr));
		fprintf(stderr, "ERROR: %s.\n", errStr);
		exit(1);
	}

	exit(0);
}
