#ifndef _LIBME9000_H_
#define _LIBME9000_H_

#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <signal.h>
#include <stdint.h>

#include "me9000mio.h"


/*=============================================================================
  General definitions
  ===========================================================================*/

/* Board numbers */
#define ME9000MIO_BOARD_0	0
#define ME9000MIO_BOARD_1	1
#define ME9000MIO_BOARD_2	2
#define ME9000MIO_BOARD_3	3

#define ME9000MIO_POINTER_NOT_USED	NULL
#define ME9000MIO_VALUE_NOT_USED	0


/*=============================================================================
  Interrupt definitions
  ===========================================================================*/

typedef void (*ME9000MIO_P_DIO_PROC)(void *pDIOIrqContext);
typedef void (*ME9000MIO_P_CNT_PROC)(void *pCntIrqContext);


/*=============================================================================
  Error definitions
  ===========================================================================*/

#define ME9000MIO_NO_ERROR				0

#define ME9000MIO_ERROR_DEFAULT_PROC_DISABLE	0
#define ME9000MIO_ERROR_DEFAULT_PROC_ENABLE		1

typedef void (*ME9000MIO_P_ERROR_PROC) (char *pcFunctionName, int iErrorCode);


#ifdef __cplusplus
extern "C" {
#endif

	/*===========================================================================
	  Error Functions
	  =========================================================================*/

	int me9000mioErrorGetMessage(
			int iErrorCode,
			char *pcBuffer,
			unsigned int uiBufferSize);

	int me9000mioErrorGetLastMessage(char *pcBuffer, unsigned int uiBufferSize);
	int me9000mioErrorSetDefaultProc(int iDefaultProcStatus);
	int me9000mioErrorSetUserProc(ME9000MIO_P_ERROR_PROC pErrorProc);


	/*===========================================================================
	  General Functions
	  =========================================================================*/

	int me9000mioGetDLLVersion(unsigned long *pulVersion);
	int me9000mioGetDriverVersion(unsigned long *pulVersion);
	int me9000mioGetBoardCount(unsigned int *puiCount);
	int me9000mioGetBoardVersion(unsigned int uiBoardNumber, unsigned short *pusVersion);
	int me9000mioGetSerialNumber(unsigned int uiBoardNumber, unsigned long *pulSerialNumber);


	/*===========================================================================
	  Digital I/O functions
	  =========================================================================*/

	int me9000mioDIOOpen(unsigned int uiBoardNumber);
	int me9000mioDIOClose(unsigned int uiBoardNumber);

	int me9000mioDIOConfig(
			unsigned int uiBoardNumber,
			unsigned int uiBitNumber,
			int iDirection,
			int iInvert,
			int iInterrupt);

	int me9000mioDIOSetCallback(
			unsigned int uiBoardNumber,
			ME9000MIO_P_DIO_PROC pDIOIrqProc,
			void *pDIOIrqContext);

	int me9000mioDIOGetBit(
			unsigned int uiBoardNumber,
			unsigned int uiBitNumber,
			int *piBitValue);

	int me9000mioDIOGetByte(unsigned int uiBoardNumber, unsigned char *pucByteValue);

	int me9000mioDIOSetBit(
			unsigned int uiBoardNumber,
			unsigned int uiBitNumber,
			int iBitValue);

	int me9000mioDIOSetByte(unsigned int uiBoardNumber, unsigned char ucByteValue);

	int me9000mioDIOOutput(
			unsigned int uiBoardNumber,
			unsigned int uiBitNumber,
			int iOutputMode);

	int me9000mioDIOGetIrqCount(unsigned int uiBoardNumber, unsigned long *pulCount);

	int me9000mioDIOReset(unsigned int uiBoardNumber);


	/*===========================================================================
	  Counter functions
	  =========================================================================*/

	int me9000mioCntOpen(unsigned int uiBoardNumber);
	int me9000mioCntClose(unsigned int uiBoardNumber);

	int me9000mioCntConfig(
			unsigned int uiBoardNumber,
			int iRetrigger,
			int iExtEvent,
			int iClockSource,
			int iInterrupt,
			ME9000MIO_P_CNT_PROC pCntIrqProc,
			void *pCntIrqContext);

	int me9000mioCntRead(unsigned int uiBoardNumber, unsigned short *pusValue);
	int me9000mioCntWrite(unsigned int uiBoardNumber, unsigned short usValue);

	int me9000mioCntStart(unsigned int uiBoardNumber);
	int me9000mioCntStop(unsigned int uiBoardNumber);

	int me9000mioCntGetIrqCount(unsigned int uiBoardNumber, unsigned long *pulCount);

	int me9000mioCntReset(unsigned int uiBoardNumber);

#ifdef __cplusplus
}
#endif

#endif
