/*
 * Copyright (C) 2004 Meilhaus Electronic GmbH (support@meilhaus.de)
 *
 * Source File : me9000mio.h
 * Author      : GG (Guenter Gebhardt)  <g.gebhardt@meilhaus.de>
 */

#ifndef _ME9000MIO_H_
#define _ME9000MIO_H_

#ifdef __KERNEL__


/*=============================================================================
  The version of the driver release
  ===========================================================================*/

#define ME9000MIO_DRIVER_VERSION  0x10100 // Version 1.01.00


/*=============================================================================
  Debug section
  ===========================================================================*/

#define ME9000MIO_CALL_DEBUG	// Debug function entry and exit
#define ME9000MIO_ISR_DEBUG	// Debug the interrupt service routine
#define ME9000MIO_PORT_DEBUG	// Debug port access
#define ME9000MIO_DEBUG		// General purpose debug masseges

// #undef ME9000MIO_CALL_DEBUG	// Debug function entry and exit
// #undef ME9000MIO_ISR_DEBUG	// Debug the interrupt service routine
// #undef ME9000MIO_PORT_DEBUG	// Debug port access
// #undef ME9000MIO_DEBUG		// General purpose debug masseges

#ifdef ME9000MIO_CALL_DEBUG
#undef CALL_PDEBUG
#define CALL_PDEBUG(fmt, args...) printk(KERN_DEBUG"ME9000MIO:" fmt, ##args)
#else
# define CALL_PDEBUG(fmt, args...) // no debugging, do nothing
#endif

#ifdef ME9000MIO_ISR_DEBUG
#undef ISR_PDEBUG
#define ISR_PDEBUG(fmt, args...) printk(KERN_DEBUG"ME9000MIO:" fmt, ##args)
#else
#define ISR_PDEBUG(fmt, args...) // no debugging, do nothing
#endif

#ifdef ME9000MIO_PORT_DEBUG
#undef PORT_PDEBUG
#define PORT_PDEBUG(fmt, args...) printk(KERN_DEBUG"ME9000MIO:" fmt, ##args)
#else
#define PORT_PDEBUG(fmt, args...) // no debugging, do nothing
#endif

#ifdef ME9000MIO_DEBUG
#undef PDEBUG
#define PDEBUG(fmt, args...) printk(KERN_DEBUG"ME9000MIO:" fmt, ##args)
#else
#define PDEBUG(fmt, args...) // no debugging, do nothing
#endif


/*=============================================================================
  PCI vendor and device IDs
  ===========================================================================*/

#define PCI_VENDOR_ID_MEILHAUS	0x1402


/*=============================================================================
  Device names, for entries in /proc/..
  ===========================================================================*/

#define ME9000MIO_NAME			"me9000mio"
#define ME9000MIO_DIO_NAME		"me9000mio_dio"
#define ME9000MIO_CNT_NAME		"me9000mio_cnt"


/*=============================================================================
  ME-9000MIO base register offsets in byte alignment
  ===========================================================================*/

#define ME9000MIO_IRQ_STATUS_REG		0x80  // R/-
#define ME9000MIO_IRQ_SOURCE_REG		0x81  // R/-

#define ME9000MIO_CNT_CNTL_REG			0x84  // R/W
#define ME9000MIO_CNT_LSB_REG			0x86  // R/W
#define ME9000MIO_CNT_MSB_REG			0x87  // R/W

#define ME9000MIO_MPIO_INT_REG			0x8F  // R/W
#define ME9000MIO_MPIO_LVL_REG			0x90  // R/W
#define ME9000MIO_MPIO_3T_REG			0x91  // R/W
#define ME9000MIO_MPIO_INV_REG			0x92  // R/W
#define ME9000MIO_MPIO_SEL_REG			0x93  // R/W (If bit is set input is selected, default is 0xFF)


/*=============================================================================
  Bits and masks for ME9000MIO_IRQ_STATUS_REG and ME9000MIO_IRQ_SOURCE_REG
  ===========================================================================*/

#define ME9000MIO_IRQ_STATUS_BIT		0x1

#define ME9000MIO_IRQ_SOURCE_MASK		0x7
#define ME9000MIO_IRQ_SOURCE_MPIO		0x6
#define ME9000MIO_IRQ_SOURCE_CNT		0x7


/*=============================================================================
  Bits for the ME9000MIO_AO_CNT_CNTL_REG
  ===========================================================================*/

#define ME9000MIO_CNT_CNTL_INT_ENABLE_BIT		0x01
#define ME9000MIO_CNT_CNTL_START_BIT			0x02
#define ME9000MIO_CNT_CNTL_ONE_SHOT_BIT			0x04
#define ME9000MIO_CNT_CNTL_EXT_CLOCK_BIT		0x08
#define ME9000MIO_CNT_CNTL_EXT_EVENT_BIT		0x10


/*=============================================================================
  Extract information from minor device number
  ===========================================================================*/

#define DIO_BOARD(dev) (MINOR(dev))
#define CNT_BOARD(dev) (MINOR(dev))


/*=============================================================================
  PCI device table.
  This is used by modprobe to translate PCI IDs to drivers.
  ===========================================================================*/

static struct pci_device_id me9000mio_pci_table[] __devinitdata = {
	{ PCI_VENDOR_ID_MEILHAUS, 0x9000, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9001, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9010, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9011, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9012, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9020, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9021, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9022, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9023, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9030, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9031, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9032, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9033, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9034, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9040, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9041, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9042, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9043, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9044, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9045, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9050, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9051, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9052, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9053, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9054, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9055, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9056, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9060, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9061, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9062, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9063, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9064, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9065, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9066, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9067, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9070, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9071, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9072, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9073, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9074, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9075, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9076, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9077, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9078, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9078, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9080, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9081, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9090, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9091, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x9092, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90A0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90A1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90A2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90A3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90B0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90B1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90B2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90B3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90B4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90C0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90C1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90C2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90C3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90C4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90C5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90D6, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E6, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90E7, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F6, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F7, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ PCI_VENDOR_ID_MEILHAUS, 0x90F8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
	{ 0 }
};
MODULE_DEVICE_TABLE(pci, me9000mio_pci_table);
#define NUMBER_ME9000_VERSIONS (sizeof(me9000mio_pci_table) / sizeof(struct pci_device_id) - 1)


/*=============================================================================
  Global board and subdevice information structures
  ===========================================================================*/

typedef struct me9000mio_info {
	struct list_head list;				// List of all detected boards
	int board_count;					// Index of the board after detection
	unsigned long me9000mio_regbase;	// Base address of the ME9000MIO
	unsigned long me9000mio_regbase_size;
	unsigned short vendor_id;			// Meilhaus vendor id (PCI_VENDOR_ID_MEILHAUS)
	unsigned short device_id;			// Device ID
	unsigned int serial_no;				// Serial number
	int pci_bus_no;						// PCI bus number
	int pci_dev_no;						// PCI device number
	int pci_func_no;					// PCI function number
	struct pci_dev *pci_dev_p;			// General PCI information
	unsigned int irq;					// IRQ assigned from the PCI BIOS
	struct me9000mio_dio_context	*dio_context;
	struct me9000mio_cnt_context	*cnt_context;
} me9000mio_info_t;


typedef struct me9000mio_dio_context {
	int in_use;                       // Indicates if already opend
	spinlock_t use_lock;              // Guards in_use
	me9000mio_info_t *board_info;
	unsigned long int_reg;
	unsigned long lvl_reg;
	unsigned long threestate_reg;
	unsigned long inv_reg;
	unsigned long sel_reg;
	unsigned long irq_status_reg;
	unsigned long irq_source_reg;
	char irq;
	unsigned long irq_count;
	struct fasync_struct *fasync_ptr;
} me9000mio_dio_context_t;


typedef struct me9000mio_cnt_context {
	int in_use;                       // Indicates if already opend
	spinlock_t use_lock;              // Guards in_use
	me9000mio_info_t *board_info;
	unsigned long cntl_reg;
	unsigned long lsb_reg;
	unsigned long msb_reg;
	unsigned long irq_status_reg;
	unsigned long irq_source_reg;
	char irq;
	unsigned long irq_count;
	struct fasync_struct *fasync_ptr;
} me9000mio_cnt_context_t;


#endif


/*=============================================================================
  Application include section starts here
  ===========================================================================*/

/*-----------------------------------------------------------------------------
  Defines for digital I/O
  ----------------------------------------------------------------------------*/

#define ME9000MIO_DIO_LINE_0			0x0
#define ME9000MIO_DIO_LINE_1			0x1
#define ME9000MIO_DIO_LINE_2			0x2
#define ME9000MIO_DIO_LINE_3			0x3
#define ME9000MIO_DIO_LINE_4			0x4
#define ME9000MIO_DIO_LINE_5			0x5
#define ME9000MIO_DIO_LINE_6			0x6
#define ME9000MIO_DIO_LINE_7			0x7

#define ME9000MIO_DIO_OUTPUT			0x0
#define ME9000MIO_DIO_INPUT				0x1

#define ME9000MIO_DIO_OUTPUT_ENABLE		0x0
#define ME9000MIO_DIO_OUTPUT_DISABLE 	0x1

#define ME9000MIO_DIO_INT_DISABLE		0x0
#define ME9000MIO_DIO_INT_ENABLE		0x1

#define ME9000MIO_DIO_INV_DISABLE		0x0
#define ME9000MIO_DIO_INV_ENABLE		0x1


/*-----------------------------------------------------------------------------
  Defines for counters
  ----------------------------------------------------------------------------*/

#define ME9000MIO_CNT_INT_DISABLE			0x0
#define ME9000MIO_CNT_INT_ENABLE			0x1

#define ME9000MIO_CNT_RETRIGGER_DISABLE		0x0
#define ME9000MIO_CNT_RETRIGGER_ENABLE		0x1

#define ME9000MIO_CNT_CLOCK_INTERNAL		0x0
#define ME9000MIO_CNT_CLOCK_EXTERNAL		0x1

#define ME9000MIO_CNT_EVENT_DISABLE			0x0
#define ME9000MIO_CNT_EVENT_ENABLE			0x1



/*-----------------------------------------------------------------------------
  General type definitions
  ----------------------------------------------------------------------------*/

typedef struct me9000mio_user_info {
	int board_count;                  // Index of the board after detection
	unsigned long me9000mio_regbase;  // Base address of the ME9000MIO
	unsigned short vendor_id;         // EXAR vendor id
	unsigned short device_id;         // Device ID
	unsigned long serial_no;          // Serial number
	int pci_bus_no;                   // PCI bus number
	int pci_dev_no;                   // PCI device number
	int pci_func_no;                  // PCI function number
	char irq;                         // IRQ assigned from the PCI BIOS
	int driver_version;               // Version of the driver release
} me9000mio_user_info_t;


/*-----------------------------------------------------------------------------
  Type definitions for digital I/O
  ----------------------------------------------------------------------------*/

typedef struct me9000mio_dio_config {
	unsigned int line;
	int dir;
	int interrupt;
	int invert;
} me9000mio_dio_config_t;

typedef struct me9000mio_dio_line {
	unsigned int line;
	int status;
} me9000mio_dio_line_t;


/*-----------------------------------------------------------------------------
  Type definitions for counters
  ----------------------------------------------------------------------------*/

typedef struct me9000mio_cnt_config {
	int retrigger;
	int ext_event;
	int interrupt;
	int clock;
} me9000mio_cnt_config_t;


/*-----------------------------------------------------------------------------
  The boards ioctls
  ----------------------------------------------------------------------------*/

#define ME9000MIO_MAGIC 'c'
#define ME9000MIO_GET_USER_INFO          _IOR (ME9000MIO_MAGIC, 0, me9000mio_user_info_t)

#define ME9000MIO_DIO_CONFIG             _IOW (ME9000MIO_MAGIC, 1, me9000mio_dio_config_t)
#define ME9000MIO_DIO_GET_LINE           _IOR (ME9000MIO_MAGIC, 2, me9000mio_dio_line_t)
#define ME9000MIO_DIO_SET_LINE           _IOW (ME9000MIO_MAGIC, 3, me9000mio_dio_line_t)
#define ME9000MIO_DIO_GET_BYTE           _IOR (ME9000MIO_MAGIC, 4, uint8_t)
#define ME9000MIO_DIO_SET_BYTE           _IOW (ME9000MIO_MAGIC, 5, uint8_t)
#define ME9000MIO_DIO_TRISTATE           _IOW (ME9000MIO_MAGIC, 6, me9000mio_dio_line_t)
#define ME9000MIO_DIO_GET_IRQ_COUNT      _IOR (ME9000MIO_MAGIC, 7, unsigned long)

#define ME9000MIO_CNT_CONFIG             _IOW (ME9000MIO_MAGIC, 10, me9000mio_cnt_config_t)
#define ME9000MIO_CNT_READ               _IOR (ME9000MIO_MAGIC, 11, uint16_t)
#define ME9000MIO_CNT_WRITE              _IOW (ME9000MIO_MAGIC, 12, uint16_t)
#define ME9000MIO_CNT_START              _IO  (ME9000MIO_MAGIC, 13)
#define ME9000MIO_CNT_STOP               _IO  (ME9000MIO_MAGIC, 14)
#define ME9000MIO_CNT_GET_IRQ_COUNT      _IOR (ME9000MIO_MAGIC, 15, unsigned long)

#endif
